// Fill out your copyright notice in the Description page of Project Settings.

#include "CharacterTPoseGenerator.h"


#define printUI(id, text) if (GEngine) GEngine->AddOnScreenDebugMessage(id, 4, FColor::Red,text)

// Sets default values for this component's properties
UCharacterTPoseGenerator::UCharacterTPoseGenerator()
{
	// Set this component to be initialized when the game starts, and to be ticked every frame.  You can turn these features
	// off to improve performance if you don't need them.
	PrimaryComponentTick.bCanEverTick = true;

	// ...
	Actor = GetOwner();
}


// Called when the game starts
void UCharacterTPoseGenerator::BeginPlay()
{
	Super::BeginPlay();

	// ...
	PrintTPose(EBoneSpaces::ComponentSpace);
}


// Called every frame
void UCharacterTPoseGenerator::TickComponent(float DeltaTime, ELevelTick TickType, FActorComponentTickFunction* ThisTickFunction)
{
	Super::TickComponent(DeltaTime, TickType, ThisTickFunction);

	// ...
}

// Initialize the model
void UCharacterTPoseGenerator::PrintTPose(EBoneSpaces::Type space)
{
	try
	{
		//find reference
		if(Actor != NULL)
		{
			StartTransform = Actor->GetActorTransform();
			
			PoseableMesh = Actor->FindComponentByClass<UPoseableMeshComponent>();
			if(PoseableMesh != NULL)
			{
				
				//print to log
				int32 NumBones = PoseableMesh->GetNumBones();
				
				UE_LOG(LogTemp, Log, TEXT("const TArray<FTransform> SegData::TPose = { //Actor:%s"), *Actor->GetHumanReadableName());
				
				for (int32 i = 0; i < NumBones; ++i)
				{
					//					EBoneSpaces space = EBoneSpaces::WorldSpace;
					FName boneName = PoseableMesh->GetBoneName(i);
					//	FTransform(FQuat().MakeFromEuler(FVector( 0.000014f, -0.000010f, 89.999985f)), FVector(216.722870f,-19.999935f,139.896027f), FVector(1.000000f, 1.000000f, 1.000000f)),		//Hips
					FTransform tr = PoseableMesh->GetBoneTransformByName(boneName, space);
					FVector pos = tr.GetLocation();
					FVector scale = tr.GetScale3D();
					FQuat rot = tr.GetRotation();
					FVector euler = rot.Euler();
					
					UE_LOG(LogTemp, Log, TEXT("FTransform(FQuat().MakeFromEuler(FVector(%f,%f,%f)), FVector(%f,%f,%f), FVector(%f,%f,%f)),  //%s")
						   , euler.X, euler.Y, euler.Z
						   , pos.X, pos.Y, pos.Z
						   , scale.X, scale.Y, scale.Z
						   , *PoseableMesh->GetBoneName(i).ToString());
				}
				
				UE_LOG(LogTemp, Log, TEXT("//"));
				UE_LOG(LogTemp, Log, TEXT("FTransform(FQuat::Identity, FVector::ZeroVector, FVector::OneVector),	//prop - reserved"));
				UE_LOG(LogTemp, Log, TEXT("FTransform(FQuat::Identity, FVector::ZeroVector, FVector::OneVector),	//prop - reserved"));
				UE_LOG(LogTemp, Log, TEXT("FTransform(FQuat::Identity, FVector::ZeroVector, FVector::OneVector),	//prop - reserved"));
				UE_LOG(LogTemp, Log, TEXT("FTransform(FQuat::Identity, FVector::ZeroVector, FVector::OneVector),	//prop - reserved"));
				UE_LOG(LogTemp, Log, TEXT("}; //Actor:%s"), *Actor->GetHumanReadableName());
			}
			else
			{
				printUI(-1, TEXT("InitPose failed: Missing posemesh!"));
			}
			
		}
	}
	catch(...)
	{
		printUI(-1, TEXT("ERROR while initing Model"));
	}
	
}
